using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using Avalonia.Controls;
namespace ReLaunch;

public class Launcher
{
    public static string GetDefaultJava()
    {

        switch (Environment.OSVersion.Platform)
        {
            case PlatformID.Win32NT:
                return Environment.GetEnvironmentVariable("JAVA_HOME", EnvironmentVariableTarget.Machine);
                break;
            case PlatformID.Unix:
                return Path.GetFullPath("/usr/lib/jvm/default-runtime/");
                break;
            default:
                return null;
                break;
        }

    }
    public static string GetLibraries(string location)
    {
        string returnValue = "";
        foreach (string file in Directory.GetFiles(Path.GetFullPath(location)))
        {
            returnValue += file + ":";
        }
        return returnValue;
    }
    public static string LaunchCommandConstructor(string javaPath, string arguments, string natives, string jar, string libraries, string entryPoint, string username)
    {
        return $"{arguments} -Djava.library.path={Path.GetFullPath(natives)} -cp \"{Path.GetFullPath(jar)}:{libraries}\" {entryPoint} {username}";
    }
    static string GetJavaExecutable()
    {
        string javaExecutable = "";

        if (RuntimeInformation.IsOSPlatform(OSPlatform.Windows))
        {
            javaExecutable = "bin\\javaw.exe";
        }
        if (RuntimeInformation.IsOSPlatform(OSPlatform.Linux) || RuntimeInformation.IsOSPlatform(OSPlatform.OSX)) //idk if its like this on osx too and im too lazy to test it
        {
            javaExecutable = "bin/java";
        }
        return javaExecutable;
    }
    public static async Task LaunchMinecraft(Settings settings, MinecraftLog logger, string nativesFolder, string jarsFolder, string librariesFolder, string minecraftFolder, string entryPoint)
    {
        await logger.LogAsync("Launching!");
        string command = LaunchCommandConstructor(settings.JavaPath, settings.Arguments, nativesFolder, $"{jarsFolder}/{settings.MinecraftJar}.jar", GetLibraries(librariesFolder), entryPoint, settings.username);
        Console.WriteLine(command);
        if (IsStringEmpty(settings.JavaPath))
        {
            settings.JavaPath = GetDefaultJava();
        }

        string javaExecutable = settings.JavaPath + GetJavaExecutable();

        using (Process minecraft = new Process())
        {
            var processInfo = new ProcessStartInfo
            {
                RedirectStandardInput = true,
                RedirectStandardOutput = true,
                RedirectStandardError = true,
                UseShellExecute = false,
                CreateNoWindow = true,
                WorkingDirectory = minecraftFolder
            };

            minecraft.StartInfo = processInfo;
            minecraft.StartInfo.FileName = settings.JavaPath + GetJavaExecutable();
            minecraft.StartInfo.Arguments = command;


            minecraft.OutputDataReceived += async (sender, e) =>
            {
                if (!string.IsNullOrEmpty(e.Data))
                {
                    await logger.LogAsync(e.Data);
                }
            };

            minecraft.ErrorDataReceived += async (sender, e) =>
            {
                if (!string.IsNullOrEmpty(e.Data))
                {
                    await logger.LogAsync(e.Data);
                }
            };

            // Start the process and begin asynchronous reading
            minecraft.Start();

            minecraft.BeginOutputReadLine();
            minecraft.BeginErrorReadLine();

            await minecraft.WaitForExitAsync();
        }
    }

    private static bool IsStringEmpty(string s)
    {
        return s == "" || s == null;
    }
}